////////////////////////////////////////////////////////////////
// JAKADAPTER (Paddlemouse version) v.1.1 for HW v.2.1
// Copyright 2012 - 2017 Jaakko Kairus
// jaakko@kair.us
//
// You may freely use and modify this software for personal use.
// Commercial use of this software or parts of it is prohibited
// without written permission from author.
// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY
//
// For more information about this project see project web page at
// http://kair.us/projects/jakadapter/
//
// Compiled with CCS compiler version 5.054
// Total compiled length 4865 bytes
////////////////////////////////////////////////////////////////
// Recent changes:
// 3.3.2015  v1.0 Initial attempt to implement paddles as HID mouse. Too much jitter,causes 'drift' in Vice.
// 5.8.2017  v1.1 Second attempt. Now based on 'normal' Jakadapter version 1.3 with greatly improved paddle
//                filtering code. Y-axis needed to be inverted for Vice. Initially seems to work quite good on Vice.

#include <18F24K50.h>
#device ADC=10
#build(stack=256)
#fuses PLL3X,INTRC,NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV28,NOPROTECT,NOLVP,NODEBUG,PLLEN,NOCPUDIV,PUT,NOMCLR
#use delay(int=16MHz, clock=48MHz, USB_FULL, act=USB)
#include <pic18_usb.h>   //Microchip PIC18Fxx5x hardware layer for usb.c
// #include <usb_desc_jakadapter.h>    //USB Configuration and Device descriptors for this USB device
#include <usb_desc_pmouse.h>    //USB Configuration and Device descriptors for this UBS device
#include <usb.c>        //handles usb setup tokens and get descriptor reports
#include <stdlib.h>
#include "jakadap21_pindef.h"	// Pin defines for HW v2.1
#include "usb_bootloader.h"		// sets program location and reset vector for use with bootloader
#import(FILE=bootloader-PIC18F24K50-dav-232.hex,HEX,RANGE=0:0x7FF)		// Embed the bootloader binary
#define WPUB 0xf85
#define MD_POLL_INT 35536 // 2.5 ms = 30000 clock cycles at 12 MHz -> MD_POLL_INT=65536-30000
#define PADDLE_FILTER_INT 36286	// 19.5 ms = 29250 clock cycles at 1.5 MHz -> PADDLE_FILTER_INT=65536-29250
#define PADDLE_FILTER_LEN 25	// paddle anti-jitter filter (max) length
#bit TMR1IF = 0xF9E.0	// Timer 1 interrupt flag for polling (not actually using an interrupt)
#bit TMR3IF = 0xFA1.1	// Timer 3 interrupt flag for polling

typedef enum {
        MODE_ATARI,
        MODE_PADDLES,
        MODE_MEGADRIVE,
        MODE_MEGADRIVE_6BTN
} MODES;
MODES mode, mode2;

unsigned int8 out_data_1[4];	// Arrays for joy1 and joy2 USB output data
unsigned int8 out_data_2[4];
unsigned int8 md_button_poll, md1_button_state, md2_button_state;	// Megadrive controller multiplexed buttons are only read at 333 Hz

unsigned int16 pot1xf[PADDLE_FILTER_LEN];	// array to store potentiometer readings for filtering
unsigned int16 pot1yf[PADDLE_FILTER_LEN];	// array to store potentiometer readings for filtering
unsigned int8 pot1xf_current=0;
unsigned int8 pot1yf_current=0;
unsigned int16 pot2xf[PADDLE_FILTER_LEN];	// array to store potentiometer readings for filtering
unsigned int16 pot2yf[PADDLE_FILTER_LEN];	// array to store potentiometer readings for filtering
unsigned int8 pot2xf_current=0;
unsigned int8 pot2yf_current=0;
unsigned int8 pollsInLen=25;				// How many USB polls are done within the paddle filter length

signed int8 pot1xf_previous=0;
signed int8 pot1yf_previous=0;
signed int8 pot2xf_previous=0;
signed int8 pot2yf_previous=0;

unsigned int8 paddle1DisconnectDelay, paddle2DisconnectDelay;

void usb_joystick_1_task(void);
void usb_paddle_task(void);
void usb_megadrive_task(void);
void usb_joystick_2_task(void);
void usb_paddle_2_task(void);
void usb_megadrive_2_task(void);
void usb_paddlemouse_1_task(void);
void usb_paddlemouse_2_task(void);
void detectPaddles(unsigned int8 adcPort, MODES &portMode, unsigned int8 &disconnectDelay);
unsigned int8 readFilteredPot(unsigned int8 adcPort, unsigned int16 filterArray[], unsigned int8 &currentIndex);

void main(void) 
{
	mode=MODE_MEGADRIVE;					// Default is Megadrive
	mode2=MODE_MEGADRIVE;					// Default is Megadrive
	usb_init();
	PORT_E_PULLUPS(TRUE);					// Activates weak pull-up on pin RE3 (VPP/MCLR)
	PORT_B_PULLUPS(0b11000011);				// Activates weak pull-up on RB0,RB1,RB6,RB7

	setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);	// Setup timer1 to use fclk/4 (12 MHz) as input
	setup_timer_3(T3_INTERNAL|T3_DIV_BY_8);	// Setup timer3 to use fclk/4 (12 MHz) as input, prescaler 8. Max period thus 65536*666 ns = 43.7 ms

	setup_dac(DAC_VSS_FVR);					// Setup the 5-bit DAC, internal only

	// drive unused pins to GND:
	OUTPUT_LOW(UNUSED1);
	OUTPUT_LOW(UNUSED2);
	OUTPUT_LOW(UNUSED3);
	#ifdef UNUSED4
	OUTPUT_LOW(UNUSED4);
	#endif

	OUTPUT_HIGH(X1_5);		// Activates +5V to megadrive pin 5 port 1
	delay_ms(5);
	OUTPUT_FLOAT(X1_7_ADD);	// Tristate additional pin 7 since select carries no current
	OUTPUT_LOW(X1_7);		// Activates GND to megadrive pin 7 port 1(select)
	delay_us(15);
	if (INPUT(X1_3)||INPUT(X1_4)) {	// Check if either left or right is high
		mode = MODE_ATARI;	// There is not Megadrive controller attached in port 1
		OUTPUT_HIGH(X1_7);	// Activate +5V to pin 7 for Atari joy/paddles
		OUTPUT_HIGH(X1_7_ADD);	// Activate additional +5V to pin 7 for more current
		OUTPUT_FLOAT(X1_5);	// Do not drive pin 5 for Atari joy/paddles
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL );  // AN11 for p1 POT_Y, AN12 for p1 POT_X, AN8 for p2 Y, AN9 for p2 X
		setup_adc(ADC_CLOCK_DIV_64);
	}	

	OUTPUT_HIGH(X2_5);		// Activates +5V to megadrive pin 5 port 2
	delay_ms(5);
	OUTPUT_FLOAT(X2_7_ADD);	// Tristate additional pin 7 since select carries no current
	OUTPUT_LOW(X2_7);		// Activates GND to megadrive pin 7 port 2(select)
	delay_us(15);
	if (INPUT(X2_3)||INPUT(X2_4)) {	// Check if either left or right is high
		mode2 = MODE_ATARI;	// There is not Megadrive controller attached in port 2
		OUTPUT_HIGH(X2_7);	// Activate +5V to pin 7 for Atari joy/paddles
		OUTPUT_HIGH(X2_7_ADD);	// Activate additional +5V to pin 7 for more current
		OUTPUT_FLOAT(X2_5);	// Do not drive pin 5 for Atari joy/paddles
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL|X2_9_ANSEL );  // AN11 for p1 POT_Y, AN12 for p1 POT_X, AN8 for p2 Y, AN9 for p2 X
		setup_adc(ADC_CLOCK_DIV_64); // Only recommended ADC divisor for 48 MHz Fosc. Equals TAD period of 1.3 s
	}	

	while (TRUE) {
		if (usb_enumerated()) {
			
			if (pollsInLen > 24)		// Wait for 230 polls for the polling rate to stabilize after enumeration
				pollsInLen++;
			if (pollsInLen == 255) {	// Start to determine USB poll rate
				set_timer3(PADDLE_FILTER_INT);
				TMR3IF=0;
				pollsInLen = 0;
			}
			if (!(TMR3IF)) {		// increase the pollsInLen until 20 ms has elapsed
				pollsInLen++;
			}
				
				
			if(TMR1IF) {				// enough time spent, megadrive buttons can be read
				set_timer1(MD_POLL_INT);
				md_button_poll=1;
				TMR1IF=0;
			} else
				md_button_poll=0;
			
			detectPaddles(X1_5_AN, mode, paddle1DisconnectDelay);
			
			if (mode == MODE_ATARI)
         		usb_joystick_1_task();
         	else if (mode == MODE_PADDLES)
//         		usb_paddle_task();
         		usb_paddlemouse_1_task();
         	else
         		usb_megadrive_task();

			out_data_1[3]=0x00; // mouse wheel, we don't use it
			while (!usb_put_packet(1,out_data_1,4,USB_DTS_TOGGLE))
			{
			}

			detectPaddles(X2_5_AN, mode2, paddle2DisconnectDelay);
			
			if (mode2 == MODE_ATARI)
         		usb_joystick_2_task();
         	else if (mode2 == MODE_PADDLES)
//         		usb_paddle_2_task();
         		usb_paddlemouse_2_task();
         	else
         		usb_megadrive_2_task();
			
//			while (!usb_put_packet(1,out_data_2,4,USB_DTS_TOGGLE))
//			{
//			}

		}
	}
}

void usb_joystick_1_task(void) 
{
	#asm
	bsf	WPUB,5
	#endasm

	if (mode2==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL );

	if(!input(X1_6)) // Joystick 1 button 1
		out_data_1[0]=1;
	else
		out_data_1[0]=0;
	if(!input(X1_9))  // Joystick 1 button 2
		out_data_1[0] = out_data_1[0] + 2;
   
	if(!input(X1_2))			// down
		out_data_1[2]=126;
	else if(!input(X1_1))		// up
		out_data_1[2]=-126;
	else
		out_data_1[2]=0;

	if(!input(X1_4))			// right
		out_data_1[1]=126;
	else if(!input(X1_3))		// left
		out_data_1[1]=-126;
	else
		out_data_1[1]=0;
	// usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_paddle_task(void) 
{
	#asm
	bcf	WPUB,5
	#endasm

	if (mode2==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_5_ANSEL );

	out_data_1[0]=0;
	if(!input(X1_3))	// Paddle button 1
		out_data_1[0] += 1;
	if(!input(X1_4))	// Paddle button 2
		out_data_1[0] += 2;

	out_data_1[1] = readFilteredPot(X1_9_AN, pot1xf, pot1xf_current);
	out_data_1[2] = readFilteredPot(X1_5_AN, pot1yf, pot1yf_current);

	// usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_megadrive_task(void) 
{
	#asm
	bsf	WPUB,5
	#endasm

	output_float(X1_7_ADD);
	out_data_1[0]=0;
	if(!input(X1_6))  // Megadrive button B
		out_data_1[0] += 2;
	if(!input(X1_9))  // Megadrive button C
		out_data_1[0] += 4;

	if(!input(X1_2))			// down
		out_data_1[2]=126;
	else if(!input(X1_1))		// up
		out_data_1[2]=-126;
	else
		out_data_1[2]=0;

	if(!input(X1_4))			// right
		out_data_1[1]=126;
    else if(!input(X1_3))		// left
		out_data_1[1]=-126;
	else
		out_data_1[1]=0;

	if(md_button_poll) {	
		output_low(X1_7); // 1st low
		delay_us(14);
		
		md1_button_state = 0;
		if(!input(X1_6))	// Megadrive button A
			md1_button_state += 1;
		if(!input(X1_9))  // Megadrive button Start
			md1_button_state += 8;
		output_high(X1_7);
		delay_us(14);
		output_low(X1_7); // 2nd low
		delay_us(15);        
		output_high(X1_7);
		delay_us(14);
		output_low(X1_7);	// 3rd low
		delay_us(14);

		if (!input(X1_1) && !input(X1_2))
			mode=MODE_MEGADRIVE_6BTN;
		
		output_high(X1_7);
		delay_us(13);
		if(!input(X1_1)&&mode==MODE_MEGADRIVE_6BTN)  // Megadrive button Z
			md1_button_state += 64;
		if(!input(X1_2)&&mode==MODE_MEGADRIVE_6BTN)  // Megadrive button Y
			md1_button_state += 32;
		if(!input(X1_3)&&mode==MODE_MEGADRIVE_6BTN)  // Megadrive button X
			md1_button_state += 16;
		if(!input(X1_4)&&mode==MODE_MEGADRIVE_6BTN)  // Megadrive button MODE
			md1_button_state += 128;
		mode=MODE_MEGADRIVE;	
	
		output_low(X1_7);
		delay_us(14);        
		output_high(X1_7);
		delay_us(14);
	}		
	out_data_1[0] += md1_button_state; //button state goes here

	// usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_paddlemouse_1_task(void) 
{
	#asm
	bcf	WPUB,5
	#endasm

	signed int8 temp;

	if (mode2==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_5_ANSEL );
	
	out_data_1[0]=0;
	if(!input(X1_3))	// Paddle button 1
		out_data_1[0] += 1;
	if(!input(X1_4))	// Paddle button 2
		out_data_1[0] += 2;

	temp = readFilteredPot(X1_9_AN, pot1xf, pot1xf_current);
	out_data_1[1] = temp - pot1xf_previous;
	pot1xf_previous = temp;

	temp = readFilteredPot(X1_5_AN, pot1yf, pot1yf_current);
	out_data_1[2] = pot1yf_previous - temp;
	pot1yf_previous = temp;
	out_data_1[3] = 0;
}

void usb_joystick_2_task(void) 
{
	#asm
	bsf	WPUB,3
	#endasm
	if (mode==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL );

	if(!input(X2_6))  // Joystick 2 button 1
		out_data_2[0]=1;
	else
		out_data_2[0]=0;
	if(!input(X2_9))  // Joystick 2 button 2
		out_data_2[0] += 2;

	if(!input(X2_2))			// down
		out_data_2[2]=126;
	else if(!input(X2_1))		// up
		out_data_2[2]=-126;
	else
		out_data_2[2]=0;

	if(!input(X2_4))			// right
		out_data_2[1]=126;
	else if(!input(X2_3))		// left
		out_data_2[1]=-126;
	else
		out_data_2[1]=0;
		
}

void usb_paddle_2_task(void) 
{
	#asm
	bcf	WPUB,3
	#endasm
	
	if (mode==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL|X2_5_ANSEL );

	out_data_2[0]=0;
	if(!input(X2_3))	// Paddle button 1
		out_data_2[0] += 1;
	if(!input(X2_4))	// Paddle button 2
		out_data_2[0] += 2;

	out_data_2[1] = readFilteredPot(X2_9_AN, pot2xf, pot2xf_current);
	out_data_2[2] = readFilteredPot(X2_5_AN, pot2yf, pot2yf_current);
}

void usb_megadrive_2_task(void) 
{
	#asm
	bsf	WPUB,3
	#endasm

	output_float(X2_7_ADD);
	
	if(!input(X2_6))  // Megadrive button B
		out_data_2[0] = 2;
	else
		out_data_2[0] = 0;
	if(!input(X2_9))  // Megadrive button C
		out_data_2[0] +=4;

	if(!input(X2_2))			// down
		out_data_2[2]=126;
	else if(!input(X2_1))		// up
		out_data_2[2]=-126;
	else out_data_2[2] = 0;

	if(!input(X2_4))			// right
		out_data_2[1]=126;
    else if(!input(X2_3))		// left
		out_data_2[1]=-126;
	else out_data_2[1] = 0;
	
	if(md_button_poll) {	
		output_low(X2_7); // 1st low
		delay_us(12);
		
		md2_button_state = 0;
		if(!input(X2_6))	// Megadrive button A
			md2_button_state += 1;
		
		if(!input(X2_9))  // Megadrive button Start
			md2_button_state += 8;
		output_high(X2_7);
		delay_us(12);
	
		output_low(X2_7); // 2nd low
		delay_us(12);        
		output_high(X2_7);
		delay_us(12);
		output_low(X2_7);	// 3rd low
		delay_us(12);

		if (!input(X2_1) && !input(X2_2))
			mode2=MODE_MEGADRIVE_6BTN;

		output_high(X2_7);
		delay_us(12);
		if(!input(X2_1)&&mode2==MODE_MEGADRIVE_6BTN)  // Megadrive button Z
			md2_button_state += 64;
		if(!input(X2_2)&&mode2==MODE_MEGADRIVE_6BTN)  // Megadrive button Y
			md2_button_state +=32;
		if(!input(X2_3)&&mode2==MODE_MEGADRIVE_6BTN)  // Megadrive button X
			md2_button_state += 16;
		if(!input(X2_4)&&mode2==MODE_MEGADRIVE_6BTN)  // Megadrive button MODE
			md2_button_state += 128;
		mode2=MODE_MEGADRIVE;
	
		output_low(X2_7);	// 4th low
		delay_us(12);        
		output_high(X2_7);
		delay_us(12);
	}
	out_data_2[0] += md2_button_state;
}

void usb_paddlemouse_2_task(void) 
{
	#asm
	bcf	WPUB,3
	#endasm
	
	signed int8 temp;

	if (mode==MODE_PADDLES)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL|X2_5_ANSEL );

	out_data_2[0]=0;
	if(!input(X2_3))	// Paddle button 1
		out_data_2[0] += 1;
	if(!input(X2_4))	// Paddle button 2
		out_data_2[0] += 2;

	temp = readFilteredPot(X2_9_AN, pot2xf, pot2xf_current);
	out_data_2[1] = temp - pot2xf_previous;
	pot2xf_previous = temp;

	temp = readFilteredPot(X2_5_AN, pot2yf, pot2yf_current);
	out_data_2[2] = pot2yf_previous - temp;
	pot2yf_previous = temp;
	out_data_2[3] = 0;
}

void detectPaddles(unsigned int8 adcPort, MODES &portMode, unsigned int8 &disconnectDelay) {
	if (portMode==MODE_ATARI || portMode==MODE_PADDLES) {				// Check if paddles are connected
		set_adc_channel (adcPort);	// POT_Y
		delay_us(20);
		unsigned int16 temp = read_adc();
		
		if (temp > 40) {		// If paddles are connected, input is higher than GND
			if (portMode == MODE_ATARI) {
				if (disconnectDelay)
					disconnectDelay--;
				else
					portMode = MODE_PADDLES;
			}
			else
				disconnectDelay=255;
		}
		else {
			if (portMode == MODE_PADDLES) {
				if (disconnectDelay)
					disconnectDelay--;
				else
					portMode = MODE_ATARI;
			}
			else
				disconnectDelay=255;	
		}		
	}
}	


// Fixed time moving interval + 11bit ADC + bad pot contact detection
unsigned int8 readFilteredPot(unsigned int8 adcPort, unsigned int16 filterArray[], unsigned int8 &currentIndex) {
	unsigned int16 adc_reading,filtertemp;
	
	if (pollsInLen >= PADDLE_FILTER_LEN)	// USB polling delay determination in progress, do not continue yet
		return 0; 
	
	dac_write(filterArray[currentIndex]/64);	// Write the previous pot reading to DAC
	set_adc_channel(DAC_CHANNEL);				// Measure it to bring the DAC cap and mux to approximately correct range
	delay_us(5);								// Some acquisition time
	read_adc();									// The purpose of reading the DAC is to prevent crosstalk between POT X and Y axes.
	set_adc_channel(adcPort);					// Now read the actual, new value

	delay_us(50);
	read_adc();									// One dummy read
	filtertemp = read_adc();					
	filtertemp += read_adc();					// Sample two times to get a virtual 11 bit reading
		
	if (filtertemp > 304) { // Do not accept readings with bad pot contact
		currentIndex++;
		if (currentIndex==pollsInLen)
			currentIndex=0;
		filterArray[currentIndex] = filtertemp;
	}	
	filtertemp = 0;
	for (int i=0; i<pollsInLen; i++) {
		filtertemp += filterArray[i];
	}
	filtertemp /= pollsInLen;
	adc_reading = (229152/filtertemp);
	adc_reading = adc_reading - 112;
	if (adc_reading>255)
		adc_reading = 255;
	return (127-(int8)adc_reading);
}		
